@@ -23,6 +23,7 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
    */
 #import <Foundation/NSObject.h>
+#import <Foundation/NSDate.h>
 
 @class NSArray;
 @class NSCondition;
@@ -59,7 +60,11 @@
  * functions is not allowed (you must call the -get and -put: methods.<br />
  * It is recommended that you create FIFOs using the -initWithName: method
  * so that you can easily use the NSUserDefaults system to adjust their
- * configurations to tests/tweak performance.
+ * configurations to tests/tweak performance.<br />
+ * While a FIFO fundamentally works on abstract items without memory
+ * management, the API provides methods for handling NSObject values
+ * threating the FIFO as a container which retains the objects until
+ * they are removed from the FIFO.
  */
 @interface	GSFIFO : NSObject
 {
@@ -113,6 +118,41 @@
  */
 - (unsigned) get: (void**)buf  count: (unsigned)count  shouldBlock: (BOOL)block;
 
+/**
+ * Reads up to count items from the FIFO into the buf. If blocking is requested
+ * and a before date is specified, the operation blocks until the specified time
+ * and returns 0 if it could not read any items. The timeout configured for the 
+ * FIFO still takes precedence.
+ */
+- (unsigned) get: (void**)buf
+           count: (unsigned)count
+     shouldBlock: (BOOL)block
+          before: (NSDate*)date;
+
+/** Reads up to count objects from the FIFO (which must contain objects
+ * or nil items) into buf and autoreleases them.<br />
+ * If block is YES, this blocks if necessary until at least one object
+ * is available, and raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.<br />
+ * Returns the number of object actually read.
+ */
+- (unsigned) getObjects: (NSObject**)buf
+                  count: (unsigned)count
+            shouldBlock: (BOOL)block;
+
+
+/**
+ * Reads up to count autoreleased objects from the FIFO into the buf.
+ * If blocking
+ * is requested and a before date is specified, the operation blocks until the
+ * specified time and returns 0 if it could not read any items. The timeout
+ * configured for the FIFO still takes precedence.
+ */
+- (unsigned) getObjects: (NSObject**)buf
+                  count: (unsigned)count
+            shouldBlock: (BOOL)block
+                 before: (NSDate*)date;
+
 /** Gets the next item from the FIFO, blocking if necessary until an
  * item is available.  Raises an exception if the FIFO is configured
  * with a timeout and it is exceeded.<br />
@@ -120,9 +160,26 @@
  */
 - (void*) get;
 
+/** Gets the next object from the FIFO (which must contain objects or
+ * nil items), blocking if necessary until an object is available.
+ * Autoreleases the object before returning it.<br />
+ * Raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.<br />
+ * Implemented using -get:count:shouldBlock:
+ */
+- (NSObject*) getObject;
+
+/** Gets the next item from the FIFO, blocking if necessary until an
+ * item is available.  Raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.<br />
+ * The returned item/object, is not autoreleased.<br />
+ * Implemented using -get:count:shouldBlock:
+ */
+- (NSObject*) getObjectRetained NS_RETURNS_RETAINED;
+
 /** <init/>
  * Initialises the receiver with the specified capacity (buffer size).<br />
- * The capacity must lie in the range from one to a million, othewrwise
+ * The capacity must lie in the range from one to a hundred million, otherwise
  * the receiver is deallocated and this method returns nil.<br />
  * If the granularity value is non-zero, it is treated as the maximum time
  * in milliseconds for which a -get or -put: operation will pause between
@@ -136,7 +193,7 @@
  * time intervals found boundaries of bands into which to categorise wait
  * time stats.  Any wait whose duration is less than the interval specified
  * in the Nth element is counted in the stat's for the Nth band.
- * If this is nil, a default set of bundaries is used.  If it is an empty
+ * If this is nil, a default set of boundaries is used.  If it is an empty
  * array then no time based stats are recorded.<br />
  * The name string is a unique identifier for the receiver and is used when
  * printing diagnostics and statistics.  If an instance with the same name
@@ -168,24 +225,66 @@
  * The GSFIFOSingleConsumerNNN boolean is NO by default.<br />
  * The GSFIFOSingleProducerNNN boolean is NO by default.<br />
  * The GSFIFOBoundariesNNN array is missing by default.<br />
+ * If no default is found for the specific named FIFO, the default set
+ * for a FIFO with an empty name is used.
  */
 - (id) initWithName: (NSString*)n;
 
+/** Writes exactly count items from buf into the FIFO, blocking if
+ * necessary until there is space for the entire write.<br />
+ * Raises an exception if the FIFO is configured with a timeout and it is
+ * exceeded, or if the count is greater than the FIFO size, or if the
+ * FIFO was not configured for multi producer or multi consumer use.<br />
+ * If rtn is YES, the method treats the buffer as containing objects
+ * which are retained as they are added.
+ */
+- (void) putAll: (void**)buf count: (unsigned)count shouldRetain: (BOOL)rtn;
+
 /** Writes up to count items from buf into the FIFO.
  * If block is YES, this blocks if necessary until at least one item
  * can be written, and raises an exception if the FIFO is configured
  * with a timeout and it is exceeded.<br />
  * Returns the number of items actually written.
  */
-- (unsigned) put: (void**)buf  count: (unsigned)count  shouldBlock: (BOOL)block;
+- (unsigned) put: (void**)buf count: (unsigned)count shouldBlock: (BOOL)block;
+
+/** Writes up to count objects from buf into the FIFO, retaining each.<br />
+ * If block is YES, this blocks if necessary until at least one object
+ * can be written, and raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.<br />
+ * Returns the number of objects actually written.
+ */
+- (unsigned) putObjects: (NSObject**)buf
+                  count: (unsigned)count
+            shouldBlock: (BOOL)block;
 
 /** Adds an item to the FIFO, blocking if necessary until there is
  * space in the buffer.  Raises an exception if the FIFO is configured
  * with a timeout and it is exceeded.br />
+ * The item may be an object (which is not retained when added) or a
+ * pointer to memory.  The ownership of the item memory should be
+ * transferred to the FIFO.<br />
  * Implemented using -put:count:shouldBlock:
  */
 - (void) put: (void*)item;
 
+/** Adds an object to the FIFO (retaining the object), blocking if
+ * necessary until there is space in the buffer.<br />
+ * Raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.br />
+ * Implemented using -put:count:shouldBlock:
+ */
+- (void) putObject: (NSObject*)item;
+
+/** Adds an object to the FIFO, blocking if necessary until there is
+ * space in the buffer.  Raises an exception if the FIFO is configured
+ * with a timeout and it is exceeded.br />
+ * The object is not retained when added, so ownership of the memory
+ * should be transferred to the FIFO.<br />
+ * Implemented using -put:count:shouldBlock:
+ */
+- (void) putObjectConsumed: (NSObject*) NS_CONSUMED item;
+
 /** Return any available statistics for the receiver.<br />
  */
 - (NSString*) stats;
@@ -208,11 +307,38 @@
  */
 - (void*) tryGet;
 
-/** Attempts to put an item into the FIFO, returning YES on success or NO
- * if the FIFO is full.<br />
+/** Checks the FIFO and returns the first available object (autoreleased)
+ * or nil if the FIFO is empty (or contains a nil object).<br />
+ * Implemented using -get:count:shouldBlock:
+ */
+- (NSObject*) tryGetObject;
+
+/**
+ * Checks the FIFO and returns a reference to the first available item 
+ * or NULL if the FIFO is empty. <br />Calling this method does
+ * <em>not</em> remove the item from the queue.
+ */
+- (void*) peek;
+
+/**
+ * Checks the FIFO and returns the an autoreleased reference to the first
+ * available object, or nil if the FIFO is empty (or contains a nil object).
+ * <br /> Calling this method does <em>not</em> remove the object from the
+ * queue.
+ */
+- (NSObject*) peekObject;
+
+/** Attempts to put an object (or nil) into the FIFO, returning YES
+ * on success or NO if the FIFO is full.<br />
  * Implemented using -put:count:shouldBlock:
  */
 - (BOOL) tryPut: (void*)item;
+
+/** Attempts to retain an object while putting it into the FIFO,
+ * returning YES on success or NO if the FIFO is full.<br />
+ * Implemented using -put:count:shouldBlock:
+ */
+- (BOOL) tryPutObject: (NSObject*)item;
 @end
 
 /** Function to efficiently get an item from a fast FIFO.<br />
