@@ -168,7 +168,10 @@ GSLinkRemove(GSListLink *link)
 }
 
 
-/** GSLinkedList manages a list of GSListLink objects.
+/** GSLinkedList manages a list of GSListLink objects.<br />
+ * The notional direction of the list is from head to tail. So the head
+ * is considered to be the first link in the list and tail is considered
+ * to be the last (head is before tail, tail is after head).
  */
 @interface	GSLinkedList : NSObject
 {
@@ -238,6 +241,8 @@ GSLinkRemove(GSListLink *link)
 
 /** Searches from list to the end looking for the first link containing
  * object (as determined by using object's [NSObject-isEqual:] method).<br />
+ * If back is YES, the search is in the direction from tail to head
+ * rather than the normal search from head to tail.<br />
  * If from is nil the list is search from head or tail as appropriate
  * to the direction in which it is searched.
  */
@@ -247,6 +252,8 @@ GSLinkedListFindEqual(NSObject *object, GSLinkedList *list,
 
 /** Searches from list to the end looking for the first link containing
  * object (as determined by direct pointer comparison).<br />
+ * If back is YES, the search is in the direction from tail to head
+ * rather than the normal search from head to tail.<br />
  * If from is nil the list is search from head or tail as appropriate
  * to the direction in which it is searched.
  */
@@ -254,7 +261,18 @@ extern GSListLink*
 GSLinkedListFindIdentical(NSObject *object, GSLinkedList *list,
   GSListLink *from, BOOL back);
 
+/** Returns the first (head) object in the list.
+ */
+static inline id
+GSLinkedListFirstObject(GSLinkedList *list)
+{
+  if (nil == list->head)
+    return nil;
+  return list->head->item;
+}
+
 /** Inserts link immediately after at.<br />
+ * If at is nil, inserts at the end of the list (link becomes tail).<br />
  * Updates the head, tail and count variables of list.<br />
  * Does not retain link.
  */
@@ -262,18 +280,31 @@ extern void
 GSLinkedListInsertAfter(GSListLink *link, GSLinkedList *list, GSListLink *at);
 
 /** Inserts link immediately before at.<br />
+ * If at is nil, inserts at the start of the list (link becomes head).<br />
  * Updates the head, tail and count variables of list.<br />
  * Does not retain link.
  */
 extern void
 GSLinkedListInsertBefore(GSListLink *link, GSLinkedList *list, GSListLink *at);
 
-/** Moves the link to the head of the list if it is not already there.
+/** Returns the last (tail) object in the list.
+ */
+static inline id
+GSLinkedListLastObject(GSLinkedList *list)
+{
+  if (nil == list->tail)
+    return nil;
+  return list->tail->item;
+}
+
+/** Moves the link to the head of the list (makes it the first object)
+ * if it is not already there.
  */
 extern void
 GSLinkedListMoveToHead(GSListLink *link, GSLinkedList *list);
 
-/** Moves the link to the tail of the list if it is not already there.
+/** Moves the link to the tail of the list (makes it the last object)
+ * if it is not already there.
  */
 extern void
 GSLinkedListMoveToTail(GSListLink *link, GSLinkedList *list);
@@ -285,4 +316,121 @@ GSLinkedListMoveToTail(GSListLink *link, GSLinkedList *list);
 extern void
 GSLinkedListRemove(GSListLink *link, GSLinkedList *list);
 
+
+
+/** This class extends GSLinkedList by providing storage for unused links
+ * and re-using those links when a new link is needed.<br />
+ * This avoids the overhead of allocating/deallocating links and provides
+ * an API more like a mutable array.
+ */
+@interface	GSLinkStore : GSLinkedList
+{
+@public
+  Class		linkClass;	/** The class used for links	*/
+  GSListLink    *free;  	/** The unused links     */
+}
+
+/** Creates an instance of a store to be used to create links using the
+ * specified class (must be a subclass of GSListLink).  If the class is
+ * nil then GSListLink is used.
+ */
++ (GSLinkStore*) storeFor: (Class)theLinkClass;
+
+/** Adds an object at the tail of the list (calls -insertObject:after:),
+ * making it the last object in the list.<br />
+ * Returns the list link that the object is stored in.
+ */
+- (GSListLink*) addObject: (id)anObject;
+
+/** Returns the first (head) object in the list or nil if the list is empty.
+ */
+- (id) firstObject;
+
+/** Inserts anObject immediately after the specified link.  If at is nil
+ * the object is inserted at the end of the list (as tail).<br />
+ * Returns the list link that the object is stored in.
+ */
+- (GSListLink*) insertObject: (id)anObject after: (GSListLink*)at;
+
+/** Inserts anObject immediately before the specified link.  If at is nil
+ * the object is inserted at the start of the list (as head).<br />
+ * Returns the list link that the object is stored in.
+ */
+- (GSListLink*) insertObject: (id)anObject before: (GSListLink*)at;
+
+/** Returns the last (tail) object in the list or nil if the list is empty.
+ */
+- (id) lastObject;
+
+/** Removes any unused links from the list (to release the memory they
+ * occupied).
+ */
+- (void) purge;
+
+/** Removes the first (head) object from the list (or does nothing if the list
+ * is empty).
+ */
+- (void) removeFirstObject;
+
+/** Removes the last (tail) object from the list (or does nothing if the list
+ * is empty).
+ */
+- (void) removeLastObject;
+
+/** Removes the object in the specified link.
+ */
+- (void) removeObjectAt: (GSListLink*)at;
+
+/** Removes the object at the specified position.
+ */
+- (void) removeObjectAtIndex: (NSUInteger)index;
+
+@end
+
+static inline void
+GSLinkStoreConsumeLink(GSLinkStore *list, GSListLink NS_CONSUMED *link)
+{
+  link->next = list->free;
+  list->free = link;
+}
+
+static inline GSListLink* NS_RETURNS_RETAINED
+GSLinkStoreProvideLink(GSLinkStore *list)
+{
+  GSListLink    *link = list->free;
+
+  if (nil == link)
+    {
+      link = [list->linkClass new];
+    }
+  else
+    {
+      list->free = link->next;
+      link->next = nil;
+    }
+  return link;
+}
+
+/** Adds the object to the list after the specified link.<br />
+ * Calls GSLinkedListInsertAfter().<br />
+ * Returns the list link that the object is stored in.
+ */
+extern GSListLink*
+GSLinkStoreInsertObjectAfter(
+  NSObject *anObject, GSLinkStore *list, GSListLink *at);
+
+/** Adds the object to the list before the specified link.<br />
+ * Calls GSLinkedListInsertBefore().<br />
+ * Returns the list link that the object is stored in.
+ */
+extern GSListLink*
+GSLinkStoreInsertObjectBefore(
+  NSObject *anObject, GSLinkStore *list, GSListLink *at);
+
+/** Removes the object held in the specified link.<br />
+ * If at is nil or is not owned by the list, this does nothing.
+ */
+extern void
+GSLinkStoreRemoveObjectAt(GSLinkStore *list, GSListLink *at);
+
 #endif
