@@ -26,7 +26,7 @@
 
 @class	GSLinkedList;
 @class	NSDate;
-@class	NSLock;
+@class	NSRecursiveLock;
 
 /** This class provides a thread pool for performing methods
  * of objects in parallel in other threads.<br />
@@ -38,6 +38,8 @@
 @interface	GSThreadPool : NSObject
 {
   NSRecursiveLock	*poolLock;
+  NSString		*poolName;
+  unsigned		created;
   BOOL			shutdown;
   BOOL			suspended;
   NSUInteger		maxThreads;
@@ -52,7 +54,7 @@
 /** Returns an instance intended for sharing between sections of code which
  * wish to make use of threading by performing operations in other threads,
  * but which don't mind operations being interleaved with those belonging to
- * oither sections of code.<br />
+ * other sections of code.<br />
  * Always returns the same instance whenever the method is called.
  */
 + (GSThreadPool*) sharedPool;
@@ -67,6 +69,10 @@
  */
 - (NSUInteger) flush;
 
+/** Returns humnan resdable pool statistics.
+ */
+- (NSString*) info;
+
 /** Returns YES if no operations are waiting to be performed, NO otherwise.
  */
 - (BOOL) isEmpty;
@@ -88,6 +94,10 @@
  */
 - (NSUInteger) maxThreads;
 
+/** Returns the name of the pool as set using the -setPoolName: method.
+ */
+- (NSString*) poolName;
+
 /** Reverses the effect of -suspend.
  */
 - (void) resume;
@@ -95,8 +105,8 @@
 /** Adds the object to the queue for which operations should be performed.<br />
  * You may add an object more than once, but that may result in the operation
  * being performed simultaneously in more than one thread.<br />
- * If the pool is configured with zero threads or zero operations,
- * this method will simply perform the operation immediately.<br />
+ * If the pool is configured with zero threads or the queue of operations is
+ * full, this method will simply perform the operation immediately.<br />
  * The operation will be performed in a context where there is an exception
  * handler set to trap exceptions, and an autorelease pool to deal with
  * autoreleased objects.
@@ -112,6 +122,10 @@
  */
 - (void) setOperations: (NSUInteger)max;
 
+/** Sets the pool name, used as a prefix for thread names.
+ */
+- (void) setPoolName: (NSString*)aName;
+
 /** Specify the maximum number of threads in the pool (the actual number
  * used may be lower than this value).<br />
  * Default is 2.<br />
